@@ -1356,6 +1356,25 @@ int dup2(int oldfd, int newfd)
 }
 #endif /* FAKEROOT_FAKENET */
 
+#ifdef __APPLE__
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6
+
+static int check_fakeroot_disabled(void *retaddr){
+  Dl_info info;
+  const char *suffix;
+  if (fakeroot_disabled || !dladdr(retaddr, &info))
+    return fakeroot_disabled;
+  suffix = strrchr(info.dli_fname, '/');
+  if (suffix && strcmp(suffix, "/CarbonCore") == 0)
+    return 1;
+  return fakeroot_disabled;
+}
+
+#define fakeroot_disabled check_fakeroot_disabled(__builtin_return_address(0))
+
+#endif
+#endif /* __APPLE__ */
+
 uid_t getuid(void){
   if (fakeroot_disabled)
     return next_getuid();
@@ -1494,6 +1513,8 @@ int setgroups(SETGROUPS_SIZE_TYPE size, const gid_t *list){
     return 0;
 }
 
+#undef fakeroot_disabled
+
 int fakeroot_disable(int new)
 {
   int old = fakeroot_disabled;
@@ -1600,6 +1621,7 @@ getattrlist(const char *path, void *attrList, void *attrBuf,
   return 0;
 }
 
+#if HAVE_FGETATTRLIST
 #ifdef __LP64__
 int
 fgetattrlist(int fd, void *attrList, void *attrBuf,
@@ -1630,4 +1652,5 @@ fgetattrlist(int fd, void *attrList, void *attrBuf,
 
   return 0;
 }
+#endif /* if HAVE_FGETATTRLIST */
 #endif /* ifdef __APPLE__ */
